@@ -66,7 +66,6 @@ namespace metaproxy_1 {
             std::list<std::string> m_targets;
             std::string m_route;
             bool m_named_result_sets;
-            int m_number_of_sets;
         };
         struct VirtualDB::Frontend {
             Frontend(Rep *rep);
@@ -92,8 +91,6 @@ namespace metaproxy_1 {
             void fixup_npr_records(ODR odr, Z_Records *records,
                                    BackendPtr b);
 
-            BackendPtr lookup_backend_from_databases(
-                std::list<std::string> databases);
             BackendPtr create_backend_from_databases(
                 std::list<std::string> databases,
                 int &error_code,
@@ -122,25 +119,12 @@ namespace metaproxy_1 {
     }
 }
 
-yf::VirtualDB::BackendPtr yf::VirtualDB::Frontend::lookup_backend_from_databases(
-    std::list<std::string> databases)
-{
-    std::list<BackendPtr>::const_iterator map_it;
-    map_it = m_backend_list.begin();
-    for (; map_it != m_backend_list.end(); map_it++)
-        if ((*map_it)->m_frontend_databases == databases)
-            return *map_it;
-    BackendPtr null;
-    return null;
-}
-
 yf::VirtualDB::BackendPtr yf::VirtualDB::Frontend::create_backend_from_databases(
     std::list<std::string> databases, int &error_code, std::string &addinfo)
 {
     BackendPtr b(new Backend);
     std::list<std::string>::const_iterator db_it = databases.begin();
 
-    b->m_number_of_sets = 0;
     b->m_frontend_databases = databases;
     b->m_named_result_sets = false;
 
@@ -289,6 +273,7 @@ void yf::VirtualDB::Frontend::search(mp::Package &package, Z_APDU *apdu_req)
         databases.push_back(req->databaseNames[i]);
 
     Sets_it sets_it = m_sets.find(req->resultSetName);
+    bool override_set = false;
     if (sets_it != m_sets.end())
     {
         // result set already exist
@@ -305,19 +290,18 @@ void yf::VirtualDB::Frontend::search(mp::Package &package, Z_APDU *apdu_req)
 
             return;
         }
-        sets_it->second.m_backend->m_number_of_sets--;
+        override_set = true;
     }
     // pick up any existing database with named result sets ..
-    // or one which has no result sets.. yet.
+    // or one if we are overriding it
     BackendPtr b; // null for now
     std::list<BackendPtr>::const_iterator map_it;
     map_it = m_backend_list.begin();
     for (; map_it != m_backend_list.end(); map_it++)
     {
         BackendPtr tmp = *map_it;
-        if (tmp->m_frontend_databases == databases &&
-            (tmp->m_named_result_sets ||
-             tmp->m_number_of_sets == 0))
+        if (mp::util::match(tmp->m_frontend_databases, databases) &&
+            (tmp->m_named_result_sets || override_set))
         {
             b = *map_it;
             break;
@@ -386,7 +370,6 @@ void yf::VirtualDB::Frontend::search(mp::Package &package, Z_APDU *apdu_req)
         Z_Records *z_records = b_resp->records;
         if (!z_records || (z_records && z_records->which == Z_Records_DBOSD))
         {
-            b->m_number_of_sets++;
             m_sets[resultSetId] = VirtualDB::Set(b, backend_setname);
             fixup_package(search_package, b);
         }
@@ -680,7 +663,7 @@ void yf::VirtualDB::Frontend::scan(mp::Package &package, Z_APDU *apdu_req)
     for (; map_it != m_backend_list.end(); map_it++)
     {
         BackendPtr tmp = *map_it;
-        if (tmp->m_frontend_databases == databases)
+        if (mp::util::match(tmp->m_frontend_databases, databases))
             break;
     }
     if (map_it != m_backend_list.end())
